# Copyright (c) 2011 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Policy Engine For Glance"""

from collections import abc

from oslo_config import cfg
from oslo_log import log as logging
from oslo_policy import policy

from glance.common import exception
from glance.domain import proxy
from glance import policies


LOG = logging.getLogger(__name__)
CONF = cfg.CONF
_ENFORCER = None


class Enforcer(policy.Enforcer):
    """Responsible for loading and enforcing rules"""

    def __init__(self, suppress_deprecation_warnings=False):
        """Init an policy Enforcer.
           :param suppress_deprecation_warnings: Whether to suppress the
                                                 deprecation warnings.
        """
        super(Enforcer, self).__init__(CONF, use_conf=True, overwrite=False)
        # NOTE(gmann): Explicitly disable the warnings for policies
        # changing their default check_str. For new RBAC, all the policy
        # defaults have been changed and warning for each policy started
        # filling the logs limit for various tool.
        # Once we move to new defaults only world then we can enable these
        # warning again.
        self.suppress_default_change_warnings = True
        if suppress_deprecation_warnings:
            self.suppress_deprecation_warnings = True
        self.register_defaults(policies.list_rules())

    def add_rules(self, rules):
        """Add new rules to the Rules object"""
        self.set_rules(rules, overwrite=False, use_conf=self.use_conf)

    def enforce(self, context, action, target, registered=True):
        """Verifies that the action is valid on the target in this context.

           :param context: Glance request context
           :param action: String representing the action to be checked
           :param target: Dictionary representing the object of the action.
           :raises: `glance.common.exception.Forbidden`
           :returns: A non-False value if access is allowed.
        """
        if registered and action not in self.registered_rules:
            raise policy.PolicyNotRegistered(action)
        try:
            return super(Enforcer, self).enforce(action, target,
                                                 context,
                                                 do_raise=True,
                                                 exc=exception.Forbidden,
                                                 action=action)
        except policy.InvalidScope:
            raise exception.Forbidden(action=action)

    def check(self, context, action, target, registered=True):
        """Verifies that the action is valid on the target in this context.

           :param context: Glance request context
           :param action: String representing the action to be checked
           :param target: Dictionary representing the object of the action.
           :returns: A non-False value if access is allowed.
        """
        if registered and action not in self.registered_rules:
            raise policy.PolicyNotRegistered(action)
        return super(Enforcer, self).enforce(action,
                                             target,
                                             context)

    def check_is_admin(self, context):
        """Check if the given context is associated with an admin role,
           as defined via the 'context_is_admin' RBAC rule.

           :param context: Glance request context
           :returns: A non-False value if context role is admin.
        """
        return self.check(context, 'context_is_admin', context.to_dict())


def get_enforcer():
    CONF([], project='glance')
    global _ENFORCER
    if _ENFORCER is None:
        _ENFORCER = Enforcer()
    return _ENFORCER


def _enforce_image_visibility(policy, context, visibility, target):
    if visibility == 'public':
        policy.enforce(context, 'publicize_image', target)
    elif visibility == 'community':
        policy.enforce(context, 'communitize_image', target)


class ImageTarget(abc.Mapping):
    SENTINEL = object()

    def __init__(self, target):
        """Initialize the object

        :param target: Object being targeted
        """
        self.target = target
        self._target_keys = [k for k in dir(proxy.Image)
                             if not k.startswith('__')
                             # NOTE(lbragstad): The locations attributes is an
                             # instance of ImageLocationsProxy, which isn't
                             # serialized into anything oslo.policy can use. If
                             # we need to use locations in policies, we need to
                             # modify how we represent those location objects
                             # before we call enforcement with target
                             # information. Omitting for not until that is a
                             # necessity.
                             if not k == 'locations'
                             if not callable(getattr(proxy.Image, k))]

    def __getitem__(self, key):
        """Return the value of 'key' from the target.

        If the target has the attribute 'key', return it.

        :param key: value to retrieve
        """
        key = self.key_transforms(key)

        value = getattr(self.target, key, self.SENTINEL)
        if value is self.SENTINEL:
            extra_properties = getattr(self.target, 'extra_properties', None)
            if extra_properties is not None:
                value = extra_properties[key]
            else:
                value = None
        return value

    def get(self, key, default=None):
        try:
            return self.__getitem__(key)
        except KeyError:
            return default

    def __len__(self):
        length = len(self._target_keys)
        length += len(getattr(self.target, 'extra_properties', {}))
        return length

    def __iter__(self):
        for key in self._target_keys:
            yield key
        for key in getattr(self.target, 'extra_properties', {}).keys():
            yield key
        for alias in ['project_id']:
            yield alias

    def key_transforms(self, key):
        transforms = {
            'id': 'image_id',
            'project_id': 'owner',
            'member_id': 'member',
        }

        return transforms.get(key, key)



# Copyright 2015 Hewlett Packard
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests healthcheck middleware."""

import http.client
import os

from glance.tests import functional
from glance.tests import utils


class HealthcheckMiddlewareTest(functional.SynchronousAPIBase):
    def setUp(self):
        super().setUp()
        self.disable_file = '/tmp/test_path'
        self.addCleanup(self._remove_disable_file)

    def _remove_disable_file(self):
        # Delete the disable file so that it does not pollute future test runs.
        try:
            os.remove(self.disable_file)
        except FileNotFoundError:
            # Should the tests fail before the "disable file" has been created,
            # this cleanup function will not be able to delete it. This should
            # not raise an exception, though, so ignore it.
            pass

    # NOTE(cyril): as /tmp/test_path is hardcoded in the paste configuration
    # for SynchronousAPIBase, if we were to add more tests, and since they
    # would be run in parallel, we would run into issues because multiple tests
    # would rely on the existence/absence of /tmp/test_path. Should we want to
    # add tests in this class in the future, we would have to pass the path to
    # our disable file at runtime and randomize it to avoid running into this
    # issue.
    @utils.skip_if_disabled
    def test_healthcheck(self):
        # First, let's see what happens without /tmp/test_path
        self.start_server(enable_cache=False)
        response = self.api_get('/healthcheck')
        self.assertEqual(b'OK', response.body)
        self.assertEqual(http.client.OK, response.status_code)

        # Then, let's check that healthcheck is disabled by file when
        # /tmp/test_path exists
        with open(self.disable_file, 'w'):
            response = self.api_get('/healthcheck')
            self.assertEqual(b'DISABLED BY FILE', response.body)
            self.assertEqual('503 Service Unavailable', response.status)



# Copyright 2021 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import oslo_policy.policy
from oslo_serialization import jsonutils

from glance.api import policy
from glance.tests import functional


TASK1 = {
    "type": "import",
    "input": {
        "import_from": "http://example.com",
        "import_from_format": "qcow2",
        "image_properties": {
            'disk_format': 'vhd',
            'container_format': 'ovf'
        }
    }
}

TASK2 = {
    "type": "api_image_import",
    "input": {
        "import_from": "http://example.com",
        "import_from_format": "qcow2",
        "image_properties": {
            'disk_format': 'vhd',
            'container_format': 'ovf'
        }
    }
}


class TestTasksPolicy(functional.SynchronousAPIBase):
    def setUp(self):
        super(TestTasksPolicy, self).setUp()
        self.policy = policy.Enforcer()

    def set_policy_rules(self, rules):
        self.policy.set_rules(
            oslo_policy.policy.Rules.from_dict(rules),
            overwrite=True)

    def start_server(self):
        with mock.patch.object(policy, 'Enforcer') as mock_enf:
            mock_enf.return_value = self.policy
            super(TestTasksPolicy, self).start_server()

    def _create_task(self, path=None, data=None, expected_code=201):
        if not path:
            path = "/v2/tasks"
        resp = self.api_post(path,
                             json=data)
        task = jsonutils.loads(resp.text)
        self.assertEqual(expected_code, resp.status_code)
        return task

    def load_data(self):
        tasks = []
        for task in [TASK1, TASK2]:
            resp = self._create_task(data=task)
            tasks.append(resp['id'])
            self.assertEqual(task['type'], resp['type'])

        return tasks

    def test_tasks_create_basic(self):
        self.start_server()
        # First make sure create tasks works with default policy
        path = '/v2/tasks'
        task = self._create_task(path=path, data=TASK1)
        self.assertEqual('import', task['type'])

        # Now disable tasks_api_access permissions and make sure any other
        # attempts fail
        self.set_policy_rules({'tasks_api_access': '!'})
        resp = self.api_post(path, json=TASK2)
        self.assertEqual(403, resp.status_code)

    def test_tasks_index_basic(self):
        self.start_server()
        # First make sure  get tasks works with default policy
        tasks = self.load_data()
        path = '/v2/tasks'
        output = self.api_get(path).json
        self.assertEqual(len(tasks), len(output['tasks']))

        # Now disable tasks_api_access permissions and make sure any other
        # attempts fail
        self.set_policy_rules({'tasks_api_access': '!'})
        resp = self.api_get(path)
        self.assertEqual(403, resp.status_code)

    def test_tasks_get_basic(self):
        self.start_server()
        # First make sure  get task works with default policy
        tasks = self.load_data()
        path = '/v2/tasks/%s' % tasks[0]
        task = self.api_get(path).json
        self.assertEqual('import', task['type'])

        # Now disable tasks_api_access permissions and make sure any other
        # attempts fail
        self.set_policy_rules({'tasks_api_access': '!'})
        path = '/v2/tasks/%s' % tasks[1]
        resp = self.api_get(path)
        self.assertEqual(403, resp.status_code)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import http.client as http

from oslo_serialization import jsonutils

from glance.tests.functional.v2 import metadef_base


class TestNamespaces(metadef_base.MetadefFunctionalTestBase):

    def setUp(self):
        super(TestNamespaces, self).setUp()
        self.start_server(enable_cache=False)

    def test_namespace_lifecycle(self):
        # Namespace should not exist
        path = '/v2/metadefs/namespaces/MyNamespace'
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Create a namespace
        path = '/v2/metadefs/namespaces'
        headers = self._headers({'content-type': 'application/json'})
        namespace_name = 'MyNamespace'
        data = {
            "namespace": namespace_name,
            "display_name": "My User Friendly Namespace",
            "description": "My description"
        }
        response = self.api_post(path, headers=headers, json=data)
        self.assertEqual(http.CREATED, response.status_code)
        namespace_loc_header = response.headers['Location']

        # Returned namespace should match the created namespace with default
        # values of visibility=private, protected=False and owner=Context
        # Tenant
        namespace = jsonutils.loads(response.text)
        checked_keys = set([
            'namespace',
            'display_name',
            'description',
            'visibility',
            'self',
            'schema',
            'protected',
            'owner',
            'created_at',
            'updated_at'
        ])
        self.assertEqual(set(namespace.keys()), checked_keys)
        expected_namespace = {
            "namespace": namespace_name,
            "display_name": "My User Friendly Namespace",
            "description": "My description",
            "visibility": "private",
            "protected": False,
            "owner": self.tenant1,
            "self": "/v2/metadefs/namespaces/%s" % namespace_name,
            "schema": "/v2/schemas/metadefs/namespace"
        }
        for key, value in expected_namespace.items():
            self.assertEqual(namespace[key], value, key)

        # Attempt to insert a duplicate
        response = self.api_post(path, headers=headers, json=data)
        self.assertEqual(http.CONFLICT, response.status_code)

        # Get the namespace using the returned Location header
        response = self.api_get(namespace_loc_header, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        namespace = jsonutils.loads(response.text)
        self.assertEqual(namespace_name, namespace['namespace'])
        self.assertNotIn('object', namespace)
        self.assertEqual(self.tenant1, namespace['owner'])
        self.assertEqual('private', namespace['visibility'])
        self.assertFalse(namespace['protected'])

        # The namespace should be mutable
        path = '/v2/metadefs/namespaces/%s' % namespace_name
        media_type = 'application/json'
        headers = self._headers({'content-type': media_type})
        namespace_name = "MyNamespace-UPDATED"
        data = {
            "namespace": namespace_name,
            "display_name": "display_name-UPDATED",
            "description": "description-UPDATED",
            "visibility": "private",  # Not changed
            "protected": True,
            "owner": self.tenant2
        }
        response = self.api_put(path, headers=headers, json=data)
        self.assertEqual(http.OK, response.status_code, response.text)

        # Returned namespace should reflect the changes
        namespace = jsonutils.loads(response.text)
        self.assertEqual('MyNamespace-UPDATED', namespace_name)
        self.assertEqual('display_name-UPDATED', namespace['display_name'])
        self.assertEqual('description-UPDATED', namespace['description'])
        self.assertEqual('private', namespace['visibility'])
        self.assertTrue(namespace['protected'])
        self.assertEqual(self.tenant2, namespace['owner'])

        # Updates should persist across requests
        path = '/v2/metadefs/namespaces/%s' % namespace_name
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        namespace = jsonutils.loads(response.text)
        self.assertEqual('MyNamespace-UPDATED', namespace['namespace'])
        self.assertEqual('display_name-UPDATED', namespace['display_name'])
        self.assertEqual('description-UPDATED', namespace['description'])
        self.assertEqual('private', namespace['visibility'])
        self.assertTrue(namespace['protected'])
        self.assertEqual(self.tenant2, namespace['owner'])

        # Deletion should not work on protected namespaces
        path = '/v2/metadefs/namespaces/%s' % namespace_name
        response = self.api_delete(path, headers=self._headers())
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Unprotect namespace for deletion
        path = '/v2/metadefs/namespaces/%s' % namespace_name
        media_type = 'application/json'
        headers = self._headers({'content-type': media_type})
        doc = {
            "namespace": namespace_name,
            "display_name": "My User Friendly Namespace",
            "description": "My description",
            "visibility": "public",
            "protected": False,
            "owner": self.tenant2
        }
        response = self.api_put(path, headers=headers, json=doc)
        self.assertEqual(http.OK, response.status_code, response.text)

        # Deletion should work. Deleting namespace MyNamespace
        path = '/v2/metadefs/namespaces/%s' % namespace_name
        response = self.api_delete(path, headers=self._headers())
        self.assertEqual(http.NO_CONTENT, response.status_code)

        # Namespace should not exist
        path = '/v2/metadefs/namespaces/MyNamespace'
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.NOT_FOUND, response.status_code)

    def _update_namespace(self, path, headers, data):
        # The namespace should be mutable
        response = self.api_put(path, headers=headers, json=data)
        self.assertEqual(http.OK, response.status_code, response.text)

        # Returned namespace should reflect the changes
        namespace = jsonutils.loads(response.text)
        expected_namespace = {
            "namespace": data['namespace'],
            "display_name": data['display_name'],
            "description": data['description'],
            "visibility": data['visibility'],
            "protected": True,
            "owner": data['owner'],
            "self": "/v2/metadefs/namespaces/%s" % data['namespace'],
            "schema": "/v2/schemas/metadefs/namespace"
        }
        namespace.pop('created_at')
        namespace.pop('updated_at')
        self.assertEqual(namespace, expected_namespace)

        # Updates should persist across requests
        path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        namespace = jsonutils.loads(response.text)
        namespace.pop('created_at')
        namespace.pop('updated_at')
        self.assertEqual(namespace, expected_namespace)

        return namespace

    def test_role_based_namespace_lifecycle(self):
        # Create public and private namespaces for tenant1 and tenant2
        path = '/v2/metadefs/namespaces'
        headers = self._headers({'content-type': 'application/json'})
        tenant_namespaces = dict()
        for tenant in [self.tenant1, self.tenant2]:
            headers['X-Tenant-Id'] = tenant
            for visibility in ['public', 'private']:
                namespace_data = {
                    "namespace": "%s_%s_namespace" % (tenant, visibility),
                    "display_name": "My User Friendly Namespace",
                    "description": "My description",
                    "visibility": visibility,
                    "owner": tenant
                }
                namespace = self.create_namespace(path, headers,
                                                  namespace_data)
                namespace = jsonutils.loads(namespace.text)
                self.assertNamespacesEqual(namespace, namespace_data)
                tenant_namespaces.setdefault(tenant, list())
                tenant_namespaces[tenant].append(namespace)

        # Check Tenant 1 and Tenant 2 will be able to see total 3 namespaces
        # (two of own and 1 public of other tenant)
        def _get_expected_namespaces(tenant):
            expected_namespaces = []
            for x in tenant_namespaces[tenant]:
                expected_namespaces.append(x['namespace'])
            if tenant == self.tenant1:
                expected_namespaces.append(
                    tenant_namespaces[self.tenant2][0]['namespace'])
            else:
                expected_namespaces.append(
                    tenant_namespaces[self.tenant1][0]['namespace'])

            return expected_namespaces

        # Check Tenant 1 and Tenant 2 will be able to see total 3 namespaces
        # (two of own and 1 public of other tenant)
        for tenant in [self.tenant1, self.tenant2]:
            path = '/v2/metadefs/namespaces'
            headers = self._headers({'X-Tenant-Id': tenant,
                                     'X-Roles': 'reader,member'})
            response = self.api_get(path, headers=headers)
            self.assertEqual(http.OK, response.status_code)
            namespaces = jsonutils.loads(response.text)['namespaces']
            expected_namespaces = _get_expected_namespaces(tenant)
            self.assertEqual(sorted(x['namespace'] for x in namespaces),
                             sorted(expected_namespaces))

        def _check_namespace_access(namespaces, tenant):
            headers = self._headers({'X-Tenant-Id': tenant,
                                     'X-Roles': 'reader,member'})
            for namespace in namespaces:
                path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
                response = self.api_get(path, headers=headers)
                if namespace['visibility'] == 'public':
                    self.assertEqual(http.OK, response.status_code)
                else:
                    self.assertEqual(http.NOT_FOUND, response.status_code)

        # Check Tenant 1 can access public namespace and cannot access private
        # namespace of Tenant 2
        _check_namespace_access(tenant_namespaces[self.tenant2],
                                self.tenant1)

        # Check Tenant 2 can access public namespace and cannot access private
        # namespace of Tenant 1
        _check_namespace_access(tenant_namespaces[self.tenant1],
                                self.tenant2)

        total_ns = tenant_namespaces[self.tenant1] \
            + tenant_namespaces[self.tenant2]
        for namespace in total_ns:
            data = {
                "namespace": namespace['namespace'],
                "display_name": "display_name-UPDATED",
                "description": "description-UPDATED",
                "visibility": namespace['visibility'],  # Not changed
                "protected": True,  # changed
                "owner": namespace["owner"]  # Not changed
            }
            path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
            headers = self._headers({
                'X-Tenant-Id': namespace['owner'],
            })
            # Update namespace should fail with non admin role
            headers['X-Roles'] = "reader,member"
            response = self.api_put(path, headers=headers, json=data)
            self.assertEqual(http.FORBIDDEN, response.status_code)

            # Should work with admin role
            headers['X-Roles'] = "admin"
            namespace = self._update_namespace(path, headers, data)

            # Deletion should fail as namespaces are protected now
            path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
            headers['X-Roles'] = "admin"
            response = self.api_delete(path, headers=headers)
            self.assertEqual(http.FORBIDDEN, response.status_code)

            # Deletion should not be allowed for non admin roles
            path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
            response = self.api_delete(
                path, headers=self._headers({
                    'X-Roles': 'reader,member',
                    'X-Tenant-Id': namespace['owner']
                }))
            self.assertEqual(http.FORBIDDEN, response.status_code)

        # Unprotect the namespaces before deletion
        headers = self._headers()
        for namespace in total_ns:
            path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
            data = {
                "namespace": namespace['namespace'],
                "protected": False,
            }
            response = self.api_put(path, headers=headers, json=data)
            self.assertEqual(http.OK, response.status_code)

        # Get updated namespace set again
        path = '/v2/metadefs/namespaces'
        response = self.api_get(path, headers=headers)
        self.assertEqual(http.OK, response.status_code)
        self.assertFalse(namespace['protected'])
        namespaces = jsonutils.loads(response.text)['namespaces']

        # Verify that deletion is not allowed for unprotected namespaces with
        # non admin role
        for namespace in namespaces:
            path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
            response = self.api_delete(
                path, headers=self._headers({
                    'X-Roles': 'reader,member',
                    'X-Tenant-Id': namespace['owner']
                }))
            self.assertEqual(http.FORBIDDEN, response.status_code)

        # Delete namespaces of all tenants
        for namespace in total_ns:
            path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
            response = self.api_delete(path, headers=headers)
            self.assertEqual(http.NO_CONTENT, response.status_code)

            # Deleted namespace should not be returned
            path = '/v2/metadefs/namespaces/%s' % namespace['namespace']
            response = self.api_get(path, headers=headers)
            self.assertEqual(http.NOT_FOUND, response.status_code)



# Copyright 2015 Intel Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os.path
import shutil
import tarfile
import tempfile
from unittest import mock

from defusedxml.ElementTree import ParseError

from glance.async_.flows import ovf_process
import glance.tests.utils as test_utils
from oslo_config import cfg


class TestOvfProcessTask(test_utils.BaseTestCase):

    def setUp(self):
        super(TestOvfProcessTask, self).setUp()
        # The glance/tests/var dir containing sample ova packages used
        # by the tests in this class
        self.test_ova_dir = os.path.abspath(os.path.join(
                                            os.path.dirname(__file__),
                                            '../../../', 'var'))
        self.tempdir = tempfile.mkdtemp()
        self.config(work_dir=self.tempdir, group="task")

        # These are the properties that we will extract from the ovf
        # file contained in a ova package
        interested_properties = (
            '{\n'
            '   "cim_pasd":  [\n'
            '      "InstructionSetExtensionName",\n'
            '      "ProcessorArchitecture"]\n'
            '}\n')
        self.config_file_name = os.path.join(self.tempdir, 'ovf-metadata.json')
        with open(self.config_file_name, 'w') as config_file:
            config_file.write(interested_properties)

        self.image = mock.Mock()
        self.image.container_format = 'ova'
        self.image.context.is_admin = True

        self.img_repo = mock.Mock()
        self.img_repo.get.return_value = self.image

    def tearDown(self):
        if os.path.exists(self.tempdir):
            shutil.rmtree(self.tempdir)

        super(TestOvfProcessTask, self).tearDown()

    def _copy_ova_to_tmpdir(self, ova_name):
        # Copies an ova package to the tempdir from which
        # it will be read by the system-under-test
        shutil.copy(os.path.join(self.test_ova_dir, ova_name), self.tempdir)
        return os.path.join(self.tempdir, ova_name)

    @mock.patch.object(cfg.ConfigOpts, 'find_file')
    def test_ovf_process_success(self, mock_find_file):
        mock_find_file.return_value = self.config_file_name

        ova_file_path = self._copy_ova_to_tmpdir('testserver.ova')
        ova_uri = 'file://' + ova_file_path

        oprocess = ovf_process._OVF_Process('task_id', 'ovf_proc',
                                            self.img_repo)
        self.assertEqual(ova_uri, oprocess.execute('test_image_id', ova_uri))

        # Note that the extracted disk image is overwritten onto the input ova
        # file
        with open(ova_file_path, 'rb') as disk_image_file:
            content = disk_image_file.read()
        # b'ABCD' is the exact contents of the disk image file
        # testserver-disk1.vmdk contained in the testserver.ova package used
        # by this test
        self.assertEqual(b'ABCD', content)
        # 'DMTF:x86:VT-d' is the value in the testerver.ovf file in the
        # testserver.ova package
        self.image.extra_properties.update.assert_called_once_with(
            {'cim_pasd_InstructionSetExtensionName': 'DMTF:x86:VT-d'})
        self.assertEqual('bare', self.image.container_format)

    @mock.patch.object(cfg.ConfigOpts, 'find_file')
    def test_ovf_process_no_config_file(self, mock_find_file):
        # Mimics a Glance deployment without the ovf-metadata.json file
        mock_find_file.return_value = None

        ova_file_path = self._copy_ova_to_tmpdir('testserver.ova')
        ova_uri = 'file://' + ova_file_path

        oprocess = ovf_process._OVF_Process('task_id', 'ovf_proc',
                                            self.img_repo)
        self.assertEqual(ova_uri, oprocess.execute('test_image_id', ova_uri))

        # Note that the extracted disk image is overwritten onto the input
        # ova file.
        with open(ova_file_path, 'rb') as disk_image_file:
            content = disk_image_file.read()
        # b'ABCD' is the exact contents of the disk image file
        # testserver-disk1.vmdk contained in the testserver.ova package used
        # by this test
        self.assertEqual(b'ABCD', content)
        # No properties must be selected from the ovf file
        self.image.extra_properties.update.assert_called_once_with({})
        self.assertEqual('bare', self.image.container_format)

    @mock.patch.object(cfg.ConfigOpts, 'find_file')
    def test_ovf_process_not_admin(self, mock_find_file):
        mock_find_file.return_value = self.config_file_name

        ova_file_path = self._copy_ova_to_tmpdir('testserver.ova')
        ova_uri = 'file://' + ova_file_path

        self.image.context.is_admin = False

        oprocess = ovf_process._OVF_Process('task_id', 'ovf_proc',
                                            self.img_repo)
        self.assertRaises(RuntimeError, oprocess.execute, 'test_image_id',
                          ova_uri)

    def test_extract_ova_not_tar(self):
        # testserver-not-tar.ova package is not in tar format
        ova_file_path = os.path.join(self.test_ova_dir,
                                     'testserver-not-tar.ova')
        iextractor = ovf_process.OVAImageExtractor()
        with open(ova_file_path, 'rb') as ova_file:
            self.assertRaises(tarfile.ReadError, iextractor.extract, ova_file)

    def test_extract_ova_no_disk(self):
        # testserver-no-disk.ova package contains no disk image file
        ova_file_path = os.path.join(self.test_ova_dir,
                                     'testserver-no-disk.ova')
        iextractor = ovf_process.OVAImageExtractor()
        with open(ova_file_path, 'rb') as ova_file:
            self.assertRaises(KeyError, iextractor.extract, ova_file)

    def test_extract_ova_no_ovf(self):
        # testserver-no-ovf.ova package contains no ovf file
        ova_file_path = os.path.join(self.test_ova_dir,
                                     'testserver-no-ovf.ova')
        iextractor = ovf_process.OVAImageExtractor()
        with open(ova_file_path, 'rb') as ova_file:
            self.assertRaises(RuntimeError, iextractor.extract, ova_file)

    def test_extract_ova_bad_ovf(self):
        # testserver-bad-ovf.ova package has an ovf file that contains
        # invalid xml
        ova_file_path = os.path.join(self.test_ova_dir,
                                     'testserver-bad-ovf.ova')
        iextractor = ovf_process.OVAImageExtractor()
        with open(ova_file_path, 'rb') as ova_file:
            self.assertRaises(ParseError, iextractor._parse_OVF, ova_file)



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

import glance_store
from oslo_config import cfg
from oslo_upgradecheck import common_checks
from oslo_upgradecheck import upgradecheck

from glance.common import removed_config
from glance.common import wsgi  # noqa

CONF = cfg.CONF

SUCCESS = upgradecheck.Code.SUCCESS
FAILURE = upgradecheck.Code.FAILURE


class Checks(upgradecheck.UpgradeCommands):
    """Programmable upgrade checks."""

    def __init__(self, *args, **kwargs):
        super(upgradecheck.UpgradeCommands).__init__(*args, **kwargs)
        removed_config.register_removed_options()

    def _check_sheepdog_store(self):
        """Check that the removed sheepdog backend store is not configured."""
        glance_store.register_opts(CONF)
        sheepdog_present = False
        if 'sheepdog' in (getattr(CONF, 'enabled_backends') or {}):
            sheepdog_present = True

        if 'sheepdog' in (getattr(CONF.glance_store, 'stores') or []):
            sheepdog_present = True

        if sheepdog_present:
            return upgradecheck.Result(
                FAILURE,
                'The "sheepdog" backend store driver has been removed, but '
                'current settings have it configured.')

        return upgradecheck.Result(SUCCESS)

    def _check_owner_is_tenant(self):
        if CONF.owner_is_tenant is False:
            return upgradecheck.Result(
                FAILURE,
                'The "owner_is_tenant" option has been removed and there is '
                'no upgrade path for installations that had this option set '
                'to False.')
        return upgradecheck.Result(SUCCESS)

    _upgrade_checks = (
        # Added in Ussuri
        ('Sheepdog Driver Removal', _check_sheepdog_store),
        # Added in Wallaby
        ('Policy File JSON to YAML Migration',
         (common_checks.check_policy_json, {'conf': CONF})),
        # Removed in Wallaby
        ('Config option owner_is_tenant removal', _check_owner_is_tenant),
    )


def main():
    try:
        return upgradecheck.main(CONF, 'glance', Checks())
    except cfg.ConfigDirNotFoundError:
        return ('ERROR: cannot read the glance configuration directory.\n'
                'Please re-run using the --config-dir <dirname> option '
                'with a valid glance configuration directory.')


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2025 RedHat Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import http.client as http
import uuid

from oslo_config import cfg
from oslo_serialization import jsonutils

from glance.tests import functional


CONF = cfg.CONF

TENANT1 = str(uuid.uuid4())
TENANT2 = str(uuid.uuid4())
TENANT3 = str(uuid.uuid4())
TENANT4 = str(uuid.uuid4())


def get_auth_header(tenant, tenant_id=None,
                    role='reader,member', headers=None):
    """Return headers to authenticate as a specific tenant.

    :param tenant: Tenant for the auth token
    :param tenant_id: Optional tenant ID for the X-Tenant-Id header
    :param role: Optional user role
    :param headers: Optional list of headers to add to
    """
    if not headers:
        headers = {}
    auth_token = 'user:%s:%s' % (tenant, role)
    headers.update({'X-Auth-Token': auth_token})
    if tenant_id:
        headers.update({'X-Tenant-Id': tenant_id})
    return headers


class TestImageMembers(functional.SynchronousAPIBase):

    def setUp(self, single_store=True, bypass_headers=False):
        super().setUp(single_store=single_store,
                      bypass_headers=bypass_headers)
        self.config(image_member_quota=10)
        self.start_server(enable_cache=False, use_fake_auth=True)

    def _headers(self, custom_headers=None):
        base_headers = {
            'X-Identity-Status': 'Confirmed',
            'X-Auth-Token': '932c5c84-02ac-4fe5-a9ba-620af0e2bb96',
            'X-User-Id': 'f9a41d13-0c13-47e9-bee2-ce4e8bfe958e',
            'X-Tenant-Id': TENANT1,
            'X-Roles': 'reader,member',
        }
        base_headers.update(custom_headers or {})
        return base_headers

    def test_image_member_lifecycle(self):

        # Image list should be empty
        path = '/v2/images'
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(0, len(images))

        owners = ['tenant1', 'tenant2', 'admin']
        visibilities = ['community', 'private', 'public', 'shared']
        image_fixture = []
        for owner in owners:
            for visibility in visibilities:
                path = '/v2/images'
                role = 'member'
                if visibility == 'public':
                    role = 'admin'
                headers = {
                    'content-type': 'application/json',
                    'X-Auth-Token': 'createuser:%s:admin' % owner,
                    'X-Roles': role,
                }
                data = {
                    'name': '%s-%s' % (owner, visibility),
                    'visibility': visibility,
                }
                response = self.api_post(path, headers=headers, json=data)
                self.assertEqual(http.CREATED, response.status_code)
                image_fixture.append(jsonutils.loads(response.text))

        # Image list should contain 6 images for tenant1
        path = '/v2/images'
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(6, len(images))

        # Image list should contain 3 images for TENANT3
        path = '/v2/images'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(3, len(images))

        # Add Image member for tenant1-shared image
        path = '/v2/images/%s/members' % image_fixture[3]['id']
        body = {'member': TENANT3}
        response = self.api_post(path, headers=get_auth_header('tenant1'),
                                 json=body)
        self.assertEqual(http.OK, response.status_code)
        image_member = jsonutils.loads(response.text)
        self.assertEqual(image_fixture[3]['id'], image_member['image_id'])
        self.assertEqual(TENANT3, image_member['member_id'])
        self.assertIn('created_at', image_member)
        self.assertIn('updated_at', image_member)
        self.assertEqual('pending', image_member['status'])

        # Image list should contain 3 images for TENANT3
        path = '/v2/images'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(3, len(images))

        # Image list should contain 0 shared images for TENANT3
        # because default is accepted
        path = '/v2/images?visibility=shared'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(0, len(images))

        # Image list should contain 4 images for TENANT3 with status pending
        path = '/v2/images?member_status=pending'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(4, len(images))

        # Image list should contain 4 images for TENANT3 with status all
        path = '/v2/images?member_status=all'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(4, len(images))

        # Image list should contain 1 image for TENANT3 with status pending
        # and visibility shared
        path = '/v2/images?member_status=pending&visibility=shared'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(1, len(images))
        self.assertEqual(images[0]['name'], 'tenant1-shared')

        # Image list should contain 0 image for TENANT3 with status rejected
        # and visibility shared
        path = '/v2/images?member_status=rejected&visibility=shared'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(0, len(images))

        # Image list should contain 0 image for TENANT3 with status accepted
        # and visibility shared
        path = '/v2/images?member_status=accepted&visibility=shared'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(0, len(images))

        # Image list should contain 0 image for TENANT3 with status accepted
        # and visibility private
        path = '/v2/images?visibility=private'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(0, len(images))

        # Image tenant2-shared's image members list should contain no members
        path = '/v2/images/%s/members' % image_fixture[7]['id']
        response = self.api_get(path, headers=get_auth_header('tenant2'))
        self.assertEqual(http.OK, response.status_code)
        body = jsonutils.loads(response.text)
        self.assertEqual(0, len(body['members']))

        # Tenant 1, who is the owner cannot change status of image member
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        body = {'status': 'accepted'}
        response = self.api_put(path, headers=get_auth_header('tenant1'),
                                json=body)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Tenant 1, who is the owner can get status of its own image member
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.OK, response.status_code)
        body = jsonutils.loads(response.text)
        self.assertEqual('pending', body['status'])
        self.assertEqual(image_fixture[3]['id'], body['image_id'])
        self.assertEqual(TENANT3, body['member_id'])

        # Tenant 3, who is the member can get status of its own status
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        body = jsonutils.loads(response.text)
        self.assertEqual('pending', body['status'])
        self.assertEqual(image_fixture[3]['id'], body['image_id'])
        self.assertEqual(TENANT3, body['member_id'])

        # Tenant 2, who not the owner cannot get status of image member
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        response = self.api_get(path, headers=get_auth_header('tenant2'))
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Tenant 3 can change status of image member
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        body = {'status': 'accepted'}
        response = self.api_put(path, headers=get_auth_header(TENANT3),
                                json=body)
        self.assertEqual(http.OK, response.status_code)
        image_member = jsonutils.loads(response.text)
        self.assertEqual(image_fixture[3]['id'], image_member['image_id'])
        self.assertEqual(TENANT3, image_member['member_id'])
        self.assertEqual('accepted', image_member['status'])

        # Image list should contain 4 images for TENANT3 because status is
        # accepted
        path = '/v2/images'
        response = self.api_get(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.OK, response.status_code)
        images = jsonutils.loads(response.text)['images']
        self.assertEqual(4, len(images))

        # Tenant 3 invalid status change
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        body = {'status': 'invalid-status'}
        response = self.api_put(path, headers=get_auth_header(TENANT3),
                                json=body)
        self.assertEqual(http.BAD_REQUEST, response.status_code)

        # Owner can Upload data to staging image
        image_id = image_fixture[3]['id']
        path = '/v2/images/%s/stage' % image_id
        headers = get_auth_header('tenant1')
        headers.update({'Content-Type': 'application/octet-stream'})
        image_data = b'YYYYY'
        response = self.api_put(path, headers=headers,
                                json=image_data)
        self.assertEqual(http.NO_CONTENT, response.status_code)

        # Tenant3: can't upload data to tenant1-shared staging image
        path = '/v2/images/%s/stage' % image_id
        image_data = b'YYYYY'
        headers.update(get_auth_header(TENANT3))
        response = self.api_put(path, headers=headers,
                                json=image_data)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Owner cannot change status of image
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        body = {'status': 'accepted'}
        response = self.api_put(path, headers=get_auth_header('tenant1'),
                                json=body)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Add Image member for tenant2-shared image
        path = '/v2/images/%s/members' % image_fixture[7]['id']
        body = {'member': TENANT4}
        response = self.api_post(path, headers=get_auth_header('tenant2'),
                                 json=body)
        self.assertEqual(http.OK, response.status_code)
        image_member = jsonutils.loads(response.text)
        self.assertEqual(image_fixture[7]['id'], image_member['image_id'])
        self.assertEqual(TENANT4, image_member['member_id'])
        self.assertIn('created_at', image_member)
        self.assertIn('updated_at', image_member)

        # Add Image member to public image
        path = '/v2/images/%s/members' % image_fixture[2]['id']
        body = {'member': TENANT2}
        response = self.api_post(path, headers=get_auth_header('tenant1'),
                                 json=body)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Add Image member to private image
        path = '/v2/images/%s/members' % image_fixture[1]['id']
        body = {'member': TENANT2}
        response = self.api_post(path, headers=get_auth_header('tenant1'),
                                 json=body)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Add Image member to community image
        path = '/v2/images/%s/members' % image_fixture[0]['id']
        body = {'member': TENANT2}
        response = self.api_post(path, headers=get_auth_header('tenant1'),
                                 json=body)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Image tenant1-shared's members list should contain 1 member
        path = '/v2/images/%s/members' % image_fixture[3]['id']
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.OK, response.status_code)
        body = jsonutils.loads(response.text)
        self.assertEqual(1, len(body['members']))

        # Admin can see any members
        path = '/v2/images/%s/members' % image_fixture[3]['id']
        response = self.api_get(path, headers=get_auth_header('tenant1',
                                                              role='admin'))
        self.assertEqual(http.OK, response.status_code)
        body = jsonutils.loads(response.text)
        self.assertEqual(1, len(body['members']))

        # Image members not found for private image not owned by TENANT 1
        path = '/v2/images/%s/members' % image_fixture[7]['id']
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Image members forbidden for public image
        path = '/v2/images/%s/members' % image_fixture[2]['id']
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertIn("Only shared images have members", response.text)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Image members forbidden for community image
        path = '/v2/images/%s/members' % image_fixture[0]['id']
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertIn("Only shared images have members", response.text)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Image members forbidden for private image
        path = '/v2/images/%s/members' % image_fixture[1]['id']
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertIn("Only shared images have members", response.text)
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Image Member Cannot delete Image membership
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        response = self.api_delete(path, headers=get_auth_header(TENANT3))
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Delete Image member
        path = '/v2/images/%s/members/%s' % (image_fixture[3]['id'], TENANT3)
        response = self.api_delete(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.NO_CONTENT, response.status_code)

        # Now the image has no members
        path = '/v2/images/%s/members' % image_fixture[3]['id']
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.OK, response.status_code)
        body = jsonutils.loads(response.text)
        self.assertEqual(0, len(body['members']))

        # Adding 11 image members should fail since configured limit is 10
        path = '/v2/images/%s/members' % image_fixture[3]['id']
        for i in range(10):
            body = {'member': str(uuid.uuid4())}
            response = self.api_post(path, headers=get_auth_header('tenant1'),
                                     json=body)
            self.assertEqual(http.OK, response.status_code)

        body = {'member': str(uuid.uuid4())}
        response = self.api_post(path, headers=get_auth_header('tenant1'),
                                 json=body)
        self.assertEqual(http.REQUEST_ENTITY_TOO_LARGE, response.status_code)

        # Get Image member should return not found for public image
        path = '/v2/images/%s/members/%s' % (image_fixture[2]['id'], TENANT3)
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Get Image member should return not found for community image
        path = '/v2/images/%s/members/%s' % (image_fixture[0]['id'], TENANT3)
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Get Image member should return not found for private image
        path = '/v2/images/%s/members/%s' % (image_fixture[1]['id'], TENANT3)
        response = self.api_get(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Delete Image member should return forbidden for public image
        path = '/v2/images/%s/members/%s' % (image_fixture[2]['id'], TENANT3)
        response = self.api_delete(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Delete Image member should return forbidden for community image
        path = '/v2/images/%s/members/%s' % (image_fixture[0]['id'], TENANT3)
        response = self.api_delete(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.FORBIDDEN, response.status_code)

        # Delete Image member should return forbidden for private image
        path = '/v2/images/%s/members/%s' % (image_fixture[1]['id'], TENANT3)
        response = self.api_delete(path, headers=get_auth_header('tenant1'))
        self.assertEqual(http.FORBIDDEN, response.status_code)


class TestMultiStoreImageMembers(TestImageMembers):

    def setUp(self):
        super().setUp(single_store=False, bypass_headers=True)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import http.client as http

import glance_store
from oslo_log import log as logging
import webob.exc

from glance.api import policy
from glance.api.v2 import images as v2_api
from glance.api.v2 import policy as api_policy
from glance.common import exception
from glance.common import utils
from glance.common import wsgi
import glance.db
import glance.gateway
from glance.i18n import _
import glance.notifier


LOG = logging.getLogger(__name__)


class Controller(object):
    def __init__(self, db_api=None, policy_enforcer=None, notifier=None,
                 store_api=None):
        self.db_api = db_api or glance.db.get_api()
        self.policy = policy_enforcer or policy.Enforcer()
        self.notifier = notifier or glance.notifier.Notifier()
        self.store_api = store_api or glance_store
        self.gateway = glance.gateway.Gateway(self.db_api, self.store_api,
                                              self.notifier, self.policy)

    @utils.mutating
    def update(self, req, image_id, tag_value):
        image_repo = self.gateway.get_repo(req.context)
        try:
            image = image_repo.get(image_id)
            api_policy.ImageAPIPolicy(req.context, image,
                                      self.policy).modify_image()
            image.tags.add(tag_value)
            image_repo.save(image)
        except exception.NotFound:
            msg = _("Image %s not found.") % image_id
            LOG.warning(msg)
            raise webob.exc.HTTPNotFound(explanation=msg)
        except exception.Forbidden:
            msg = _("Not allowed to update tags for image %s.") % image_id
            LOG.warning(msg)
            raise webob.exc.HTTPForbidden(explanation=msg)
        except exception.Invalid as e:
            msg = (_("Could not update image: %s") % e)
            LOG.warning(msg)
            raise webob.exc.HTTPBadRequest(explanation=msg)
        except exception.ImageTagLimitExceeded as e:
            msg = (_("Image tag limit exceeded for image %(id)s: %(e)s:")
                   % {"id": image_id,
                      "e": e})
            LOG.warning(msg)
            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)

    @utils.mutating
    def delete(self, req, image_id, tag_value):
        image_repo = self.gateway.get_repo(req.context)
        try:
            image = image_repo.get(image_id)
            api_policy.ImageAPIPolicy(req.context, image,
                                      self.policy).modify_image()

            if tag_value not in image.tags:
                raise webob.exc.HTTPNotFound()
            image.tags.remove(tag_value)
            image_repo.save(image)
        except exception.NotFound:
            msg = _("Image %s not found.") % image_id
            LOG.warning(msg)
            raise webob.exc.HTTPNotFound(explanation=msg)
        except exception.Forbidden:
            msg = _("Not allowed to delete tags for image %s.") % image_id
            LOG.warning(msg)
            raise webob.exc.HTTPForbidden(explanation=msg)


class ResponseSerializer(wsgi.JSONResponseSerializer):
    def update(self, response, result):
        response.status_int = http.NO_CONTENT

    def delete(self, response, result):
        response.status_int = http.NO_CONTENT


class RequestDeserializer(wsgi.JSONRequestDeserializer):
    def update(self, request):
        try:
            schema = v2_api.get_schema()
            schema_format = {"tags": [request.urlvars.get('tag_value')]}
            schema.validate(schema_format)
        except exception.InvalidObject as e:
            raise webob.exc.HTTPBadRequest(explanation=e.msg)
        return super(RequestDeserializer, self).default(request)


def create_resource():
    """Images resource factory method"""
    serializer = ResponseSerializer()
    deserializer = RequestDeserializer()
    controller = Controller()
    return wsgi.Resource(controller, deserializer, serializer)



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import os

import glance_store as store_api
from oslo_config import cfg
from oslo_log import log as logging
from taskflow.patterns import linear_flow as lf
from taskflow import task
from taskflow.types import failure

from glance.common import exception
from glance.i18n import _, _LE

LOG = logging.getLogger(__name__)

CONF = cfg.CONF


class _CopyImage(task.Task):

    default_provides = 'file_uri'

    def __init__(self, task_id, task_type, image_repo, action_wrapper):
        self.task_id = task_id
        self.task_type = task_type
        self.image_repo = image_repo
        self.image_id = action_wrapper.image_id
        self.action_wrapper = action_wrapper
        super(_CopyImage, self).__init__(
            name='%s-CopyImage-%s' % (task_type, task_id))

        self.staging_store = store_api.get_store_from_store_identifier(
            'os_glance_staging_store')

    def execute(self):
        with self.action_wrapper as action:
            return self._execute(action)

    def _execute(self, action):
        """Create temp file into store and return path to it

        :param action: Action wrapper
        """
        # NOTE (abhishekk): If ``all_stores_must_succeed`` is set to True
        # and copying task fails then we keep data in staging area as it
        # is so that if second call is made to copy the same image then
        # no need to copy the data in staging area again.
        file_path = "%s/%s" % (getattr(
            CONF, 'os_glance_staging_store').filesystem_store_datadir,
            self.image_id)

        if os.path.exists(file_path):
            # NOTE (abhishekk): If previous copy-image operation is failed
            # due to power failure, network failure or any other reason and
            # the image data here is partial then clear the staging area and
            # re-stage the fresh image data.
            # Ref: https://bugs.launchpad.net/glance/+bug/1885003
            size_in_staging = os.path.getsize(file_path)
            if action.image_size == size_in_staging:
                return file_path, 0
            else:
                LOG.debug(("Found partial image data in staging "
                           "%(fn)s, deleting it to re-stage "
                           "again"), {'fn': file_path})
                try:
                    os.unlink(file_path)
                except OSError as e:
                    LOG.error(_LE("Deletion of staged "
                                  "image data from %(fn)s has failed because "
                                  "[Errno %(en)d]"), {'fn': file_path,
                                                      'en': e.errno})
                    raise

        # At first search image in default_backend
        default_store = CONF.glance_store.default_backend
        for loc in action.image_locations:
            if loc['metadata'].get('store') == default_store:
                try:
                    return self._copy_to_staging_store(loc)
                except store_api.exceptions.NotFound:
                    msg = (_LE("Image not present in default store, searching "
                               "in all glance-api specific available "
                               "stores"))
                    LOG.error(msg)
                    break

        available_backends = CONF.enabled_backends
        for loc in action.image_locations:
            image_backend = loc['metadata'].get('store')
            if (image_backend in available_backends.keys()
                    and image_backend != default_store):
                try:
                    return self._copy_to_staging_store(loc)
                except store_api.exceptions.NotFound:
                    LOG.error(_LE('Image: %(img_id)s is not present in store '
                                  '%(store)s.'),
                              {'img_id': self.image_id,
                               'store': image_backend})
                    continue

        raise exception.NotFound(_("Image not found in any configured "
                                   "store"))

    def _copy_to_staging_store(self, loc):
        store_backend = loc['metadata'].get('store')
        image_data, size = store_api.get(loc['url'], store_backend)
        msg = ("Found image, copying it in staging area")
        LOG.debug(msg)
        return self.staging_store.add(self.image_id, image_data, size)[0]

    def revert(self, result, **kwargs):
        if isinstance(result, failure.Failure):
            LOG.error(_LE('Task: %(task_id)s failed to copy image '
                          '%(image_id)s.'),
                      {'task_id': self.task_id,
                       'image_id': self.image_id})


def get_flow(**kwargs):
    """Return task flow for web-download.

    :param task_id: Task ID.
    :param task_type: Type of the task.
    :param image_repo: Image repository used.
    :param image_id: Image ID.
    :param action_wrapper: An api_image_import.ActionWrapper.
    """
    task_id = kwargs.get('task_id')
    task_type = kwargs.get('task_type')
    image_repo = kwargs.get('image_repo')
    action_wrapper = kwargs.get('action_wrapper')

    return lf.Flow(task_type).add(
        _CopyImage(task_id, task_type, image_repo, action_wrapper),
    )
